@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 #
-# Copyright 2018, Intel Corporation
+# Copyright 2018-2019, Intel Corporation
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions
@@ -38,7 +38,8 @@ set -e
 
 git clone https://github.com/pmem/pmdk
 cd pmdk
-git checkout 1.5.1
+# stable-1.6: common: fix typo
+git checkout d526eb00eade98ff1caa283751c2d2dc9cf276fb
 
 sudo make EXTRA_CFLAGS="-DUSE_COW_ENV" -j2 install prefix=/opt/pmdk
 
